#!/usr/bin/python3
# -*- coding: utf-8 -*-
# pilcharconv for Linux
#
# (c) 2015 Joachim Siebold
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# character conversion ---------------------------------------
#
# Changelog
#
# 19.02.2016 jsi:
# - added stringconv function
# 16.09.2017 jsi:
# - HP Roman 8 character conversion code table implemented. Use control pictures for
#   C0 control codes.
# - renamed stringconv to barrrconv. Added stringconv.
# 27.12.2018 jsi
# - added HP2225 charset
# 16.01.2019 jsi
# - refactoring: use list of lists for code tables to speed up access
# - added HP-75 charset
# - added icharconv function
# 25.01.2019 jsi
# - various bug fixes in the HP-75 and HP-71 character table
# 11.02.2019 jsi
# - fix in HP-75 character set
#

#
# Note: if the order of character sets is changed mind to alter the 
#       CHAR_ATTRIB look up table in pilqterm. All character sets
#       which are not user configurable must be ranked at last
#
CHARSET_HP71=0
CHARSET_HP41=1
CHARSET_HP75=2
CHARSET_ROMAN8=3
CHARSET_HP2225=4
#
# do not use the HP2555 character set in menus it is only used internally
#
charsets= [ "HP-71", "HP-41", "HP-75", "ROMAN-8" ]

convert_to_unicode= [
# map hp-71 character set to unicode
[
 ' ',       # 0 
 '°',       # 1
 'x',       # 2
 '\u2190' , # 3
 '\u03B1' , # 4
 '\u03B2' , # 5
 '\u0393' , # 6
 '\u2618' , # bell!
 chr(8),
 '\u03C3' ,
 chr(10),
 '\u03BB' ,
 '\u03Bc' ,
 chr(13),
 '\u03C4' ,
 '\u03A6' ,
 '\u03B8' ,
 '\u03A9' ,
 '\u03B4' ,
 '\u03B5' ,
 '\u03C0',
 'Ä' ,
 'ä' ,
 'Ö' ,
 'ö' ,
 'Ü' ,
 'ü',
 chr(27),
 '\u03A3' ,
 '\u2260' ,
 '£',
 '\u25AF' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '\'' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' ,
 '}' ,
 '~' ,
 '\u251C' ,
 ' ',       # 128
 '°',       # 129
 'x',       # 130
 '\u2190' , # 131
 '\u03B1' , # 132
 '\u03B2' , # 133
 '\u0393' , # 134
 '\u2618' , # 135
 '\u2408' , # 136 carriage return digraph
 '\u03C3' , # 137
 '\u240A' , # 138 line feed digraph
 '\u03BB' , # 139
 '\u03Bc' , # 140
 '\u21B5' , # 141 downwards arrow
 '\u03C4' ,
 '\u03A6' ,
 '\u03B8' ,
 '\u03A9' ,
 '\u03B4' ,
 '\u03B5' ,
 '\u03C0',
 'Ä' ,
 'ä' ,
 'Ö' ,
 'ö' ,
 'Ü' ,
 'ü',
 '\u25AF' ,
 '\u03A3' ,
 '\u2260' ,
 '£',
 '\u25AF' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '\'' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' ,
 '}' ,
 '~' ,
 '\u251C' 
],
# map HP-41 character code to unicode
[
 '\u25C6', 
 '\u03F0', 
 'x',
 '\u2190' ,
 '\u03B1' ,
 '\u03B2' ,
 '\u03B3' ,
 '\u2193' , 
 '\u03B4',
 '\u03A3' ,
 chr(10),
 '\u03BB' ,
 '\u03Bc' ,
 chr(13),
 '\u03C4' ,
 '\u03A6' ,
 '\u03B8' ,
 '\u03A9' ,
 '\u03B4' ,
 '\u00C5' ,
 '\u00E5',
 'Ä' ,
 'ä' ,
 'Ö' ,
 'ö' ,
 'Ü' ,
 'ü',
 '\u00C6',
 '\u00E6' ,
 '\u2260' ,
 '£',
 '\u25AF' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '\u2191' ,
 '_' ,
 '\\' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '\u03C0' ,
 '\u2280' , 
 '\u2190' , 
 '\u03A3' , 
 '\u251C' ,
 '\u25C6', 
 '\u03F0', 
 'x',
 '\u2190' ,
 '\u03B1' ,
 '\u03B2' ,
 '\u03B3' ,
 '\u2193' , 
 '\u03B4',
 '\u03A3' ,
 chr(10),
 '\u03BB' ,
 '\u03Bc' ,
 chr(13),
 '\u03C4' ,
 '\u03A6' ,
 '\u03B8' ,
 '\u03A9' ,
 '\u03B4' ,
 '\u00C5' ,
 '\u00E5',
 'Ä' ,
 'ä' ,
 'Ö' ,
 'ö' ,
 'Ü' ,
 'ü',
 '\u00C6',
 '\u00E6' ,
 '\u2260' ,
 '£',
 '\u25AF' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '\'' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' ,
 '}' ,
 '~' ,
 '\u251C' 
], 
# map HP-75 character set to unicode
 [
 '\u0394',  # 0 
 '°',       # 1
 'x',       # 2
 '\u2190' , # 3
 '\u03B1' , # 4
 '\u03B2' , # 5
 '\u0393' , # 6 capital gamma
 '\u2618' , # bell!
 chr(8),
 '\u03C3' , # 9
 chr(10),
 '\u03BB' ,
 '\u03Bc' ,
 chr(13),
 '\u03C4' ,
 '\u03A6' ,
 '\u03B8' ,
 '\u03A9' ,
 '\u03B4' ,
 '\u03B5' ,
 '\u03C0',
 'Ä' ,
 'ä' ,
 'Ö' ,
 'ö' ,
 'Ü' ,
 'ü',
 chr(27),
 '\u03A3' ,
 '\u2260' ,
 '£',
 '\u25AF' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '\'' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' ,
 '}' ,
 '~' ,
 '\u251C' , # 127, append
 '\u0394',  # 128
 '°',       # 129
 'x',       # 130
 '\u2190' , # 131
 '\u03B1' , # 132
 '\u03B2' , # 133
 '\u0393' , # 134, capital gamma
 '\u2618' , # 135
 '~'      , # 136
 '\u03C3' , # 137
 '\u251C' , # 138
 '\u03BB' , # 139
 '\u03Bc' , # 140
 '\u25AF' , # 141
 '\u03C4' , # 142
 '\u03A6' , # 143
 '\u03B8' , # 144
 '\u03C9' , # 145
 '\u03B4' , # 146
 '\u03B5' , # 147
 '\u03C0',  # 148
 'Ä' ,
 'ä' ,
 'Ö' ,
 'ö' ,
 'Ü' ,
 'ü',
 '\u241B' ,
 '\u03A3' ,
 '\u2260' ,
 '£',
 '\u25AF' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '\'' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' ,
 '}' ,
 '~' ,
 '\u251C' # 255
],  
# map ROMAN-8 to unicode
[
 '\u2400', 
 '\u2401', 
 '\u2402',
 '\u2403',
 '\u2404',
 '\u2405',
 '\u2406',
 '\u2407', 
 chr(8),    # BS
 '\u2409',
 chr(10),  # LF
 '\u240B',
 '\u240C',
 chr(13),  # CR
 '\u240E' ,
 '\u240F' ,
 '\u2410' ,
 '\u2411' ,
 '\u2412' ,
 '\u2413' ,
 '\u2414',
 '\u2415' ,
 '\u2416' ,
 '\u2417' ,
 '\u2418' ,
 '\u2419' ,
 '\u241A',
 '\u241B',
 '\u241C' ,
 '\u241D' ,
 '\u241E',
 '\u241F' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '`' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' , 
 '}' , 
 '~' , 
 chr(127) , # DEL
 '\u2426',  # undefined
 '\u2426',  # undefined
 '\u2426',
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' , 
 '\u2426',
 '\u2426' ,
 '\u2426',
 '\u2426' ,
 '\u2426' ,
 '\u2426',
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426',
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2426' ,
 '\u2423' , # NBSP
 '\u00C0' ,
 '\u00C2' ,
 '\u00C8' ,
 '\u00CA' ,
 '\u00CB' ,
 '\u00CE' ,
 '\u00CF' ,
 '\u00B4' ,
 '\u0060' ,
 '\u02C6' ,
 '\u00A8' ,
 '\u02DC' ,
 '\u00D9' ,
 '\u00DB' ,
 '\u20A4' ,
 '\u203E' ,
 '\u00DD' ,
 '\u00FD' ,
 '\u02DA' ,
 '\u00C7' ,
 '\u00E7' ,
 '\u00D1' ,
 '\u00F1' ,
 '\u00A1' ,
 '\u00BF' ,
 '\u00A4' ,
 '\u00A3' ,
 '\u00A5' ,
 '\u00A7' ,
 '\u0192' ,
 '\u00A2' ,
 '\u00E2' ,
 '\u00EA' ,
 '\u00F4' ,
 '\u00FB' ,
 '\u00E1' ,
 '\u00E9' ,
 '\u00F3' ,
 '\u00FA' ,
 '\u00E0' ,
 '\u00E8' ,
 '\u00F2' ,
 '\u00F9' ,
 '\u00E4' ,
 '\u00EB' ,
 '\u00F6' ,
 '\u00FC' ,
 '\u00C5' ,
 '\u00EE' ,
 '\u00D8' ,
 '\u00C6' ,
 '\u00E5' ,
 '\u00ED' ,
 '\u00F8' ,
 '\u00E6' ,
 '\u00C4' ,
 '\u00EC' ,
 '\u00D6' ,
 '\u00DC' ,
 '\u00C9' ,
 '\u00EF' ,
 '\u00DF' ,
 '\u00D4' ,
 '\u00C1' ,
 '\u00C3' ,
 '\u00E3' ,
 '\u00D0' ,
 '\u00F0' ,
 '\u00CD' ,
 '\u00CC' ,
 '\u00D3' ,
 '\u00D2' ,
 '\u00D5' ,
 '\u00F5' ,
 '\u0160' ,
 '\u0161' ,
 '\u00DA' ,
 '\u0178' ,
 '\u00FF' ,
 '\u00DE' ,
 '\u00FE' ,
 '\u00B7' ,
 '\u00B5' ,
 '\u00B6' ,
 '\u00BE' ,
 '\u002D' ,
 '\u00BC' ,
 '\u00BD' ,
 '\u00AA' ,
 '\u00BA' ,
 '\u00AB' ,
 '\u25A0' ,
 '\u00BB' ,
 '\u00B1' ,
 '\u2426' 
],
# - map HP2225 charset to unicode
# - bs, cr, lf and del are printable unicode symbols
# - character 0x80-0x9f are the same as 0x00-0x1f
#
[
 '\u2400', 
 '\u2401', 
 '\u2402',
 '\u2403',
 '\u2404',
 '\u2405',
 '\u2406',
 '\u2407', 
 '\u2408',
 '\u2409',
 '\u240A',
 '\u240B',
 '\u240C',
 '\u240D',
 '\u240E' ,
 '\u240F' ,
 '\u2410' ,
 '\u2411' ,
 '\u2412' ,
 '\u2413' ,
 '\u2414',
 '\u2415' ,
 '\u2416' ,
 '\u2417' ,
 '\u2418' ,
 '\u2419' ,
 '\u241A',
 '\u241B',
 '\u241C' ,
 '\u241D' ,
 '\u241E',
 '\u241F' ,
 ' ' ,
 '!' ,
 '"' ,
 '#' ,
 '$' ,
 '%' ,
 '&' ,
 '\'' ,
 '(' ,
 ')' ,
 '*' ,
 '+' ,
 ',' ,
 '-' ,
 '.' ,
 '/' ,
 '0' ,
 '1' ,
 '2' ,
 '3' ,
 '4' ,
 '5' ,
 '6' ,
 '7' ,
 '8' ,
 '9' ,
 ':' ,
 ';' ,
 '<' ,
 '=' ,
 '>' ,
 '?' ,
 '@' ,
 'A' ,
 'B' ,
 'C' ,
 'D' ,
 'E' ,
 'F' ,
 'G' ,
 'H' ,
 'I' ,
 'J' ,
 'K' ,
 'L' ,
 'M' ,
 'N' ,
 'O' ,
 'P' ,
 'Q' ,
 'R' ,
 'S' ,
 'T' ,
 'U' ,
 'V' ,
 'W' ,
 'X' ,
 'Y' ,
 'Z' ,
 '[' ,
 '\\' ,
 ']' ,
 '^' ,
 '_' ,
 '`' ,
 'a' ,
 'b' ,
 'c' ,
 'd' ,
 'e' ,
 'f' ,
 'g' ,
 'h' ,
 'i' ,
 'j' ,
 'k' ,
 'l' ,
 'm' ,
 'n' ,
 'o' ,
 'p' ,
 'q' ,
 'r' ,
 's' ,
 't' ,
 'u' ,
 'v' ,
 'w' ,
 'x' ,
 'y' ,
 'z' ,
 '{' ,
 '|' , 
 '}' , 
 '~' , 
 '\u2421',
 '\u2400', 
 '\u2401', 
 '\u2402',
 '\u2403',
 '\u2404',
 '\u2405',
 '\u2406',
 '\u2407', 
 '\u2408',
 '\u2409',
 '\u240A',
 '\u240B',
 '\u240C',
 '\u240D',
 '\u240E' ,
 '\u240F' ,
 '\u2410' ,
 '\u2411' ,
 '\u2412' ,
 '\u2413' ,
 '\u2414',
 '\u2415' ,
 '\u2416' ,
 '\u2417' ,
 '\u2418' ,
 '\u2419' ,
 '\u241A',
 '\u241B',
 '\u241C' ,
 '\u241D' ,
 '\u241E',
 '\u241F' ,
 '\u2423' , # NBSP
 '\u00C0' ,
 '\u00C2' ,
 '\u00C8' ,
 '\u00CA' ,
 '\u00CB' ,
 '\u00CE' ,
 '\u00CF' ,
 '\u00B4' ,
 '\u0060' ,
 '\u02C6' ,
 '\u00A8' ,
 '\u02DC' ,
 '\u00D9' ,
 '\u00DB' ,
 '\u20A4' ,
 '\u203E' ,
 '\u00DD' ,
 '\u00FD' ,
 '\u02DA' ,
 '\u00C7' ,
 '\u00E7' ,
 '\u00D1' ,
 '\u00F1' ,
 '\u00A1' ,
 '\u00BF' ,
 '\u00A4' ,
 '\u00A3' ,
 '\u00A5' ,
 '\u00A7' ,
 '\u0192' ,
 '\u00A2' ,
 '\u00E2' ,
 '\u00EA' ,
 '\u00F4' ,
 '\u00FB' ,
 '\u00E1' ,
 '\u00E9' ,
 '\u00F3' ,
 '\u00FA' ,
 '\u00E0' ,
 '\u00E8' ,
 '\u00F2' ,
 '\u00F9' ,
 '\u00E4' ,
 '\u00EB' ,
 '\u00F6' ,
 '\u00FC' ,
 '\u00C5' ,
 '\u00EE' ,
 '\u00D8' ,
 '\u00C6' ,
 '\u00E5' ,
 '\u00ED' ,
 '\u00F8' ,
 '\u00E6' ,
 '\u00C4' ,
 '\u00EC' ,
 '\u00D6' ,
 '\u00DC' ,
 '\u00C9' ,
 '\u00EF' ,
 '\u00DF' ,
 '\u00D4' ,
 '\u00C1' ,
 '\u00C3' ,
 '\u00E3' ,
 '\u00D0' ,
 '\u00F0' ,
 '\u00CD' ,
 '\u00CC' ,
 '\u00D3' ,
 '\u00D2' ,
 '\u00D5' ,
 '\u00F5' ,
 '\u0160' ,
 '\u0161' ,
 '\u00DA' ,
 '\u0178' ,
 '\u00FF' ,
 '\u00DE' ,
 '\u00FE' ,
 '\u00B7' ,
 '\u00B5' ,
 '\u00B6' ,
 '\u00BE' ,
 '\u002D' ,
 '\u00BC' ,
 '\u00BD' ,
 '\u00AA' ,
 '\u00BA' ,
 '\u00AB' ,
 '\u25A0' ,
 '\u00BB' ,
 '\u00B1' ,
 '\u2426' 
]
]
#
# convert single character
#
def charconv(s, charset):
   return convert_to_unicode[charset][ord(s)]
#
# convert an integer character code 
#
def icharconv(i, charset):
   return convert_to_unicode[charset][i]
#
# convert a bytearray to a string
#
def barrconv(barr, charset):
   slist= []
   for c in barr:
      slist.append(convert_to_unicode[charset][c])
   return "".join(slist)
#
# convert a string
#
def stringconv(string, charset):
   slist= []
   for c in string:
      slist.append(convert_to_unicode[charset][ord(c)])
   return "".join(slist)

